@@ -1,6 +1,6 @@
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // Mobius Forensic Toolkit
-// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Eduardo Aguiar
+// Copyright (C) 2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019,2020 Eduardo Aguiar
 //
 // This program is free software; you can redistribute it and/or modify it
 // under the terms of the GNU General Public License as published by the
@@ -15,9 +15,11 @@
 // You should have received a copy of the GNU General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-#include <mobius/imagefile/imagefile.h>
 #include <mobius/core/application.h>
-#include <mobius/uri.h>
+#include <mobius/imagefile/imagefile.h>
+#include <mobius/imagefile/imagefile_ewf.h>
+#include <mobius/imagefile/imagefile_split.h>
+#include <mobius/io/uri.h>
 #include <mobius/string_functions.h>
 #include <iostream>
 #include <cstdint>
@@ -41,7 +43,7 @@ usage ()
   std::cerr << "     input types are:" << std::endl;
   std::cerr << "       autodetect\ttry to autodetect imagefile type (default)" << std::endl;
 
-  for (auto i : mobius::imagefile::imagefile::typelist)
+  for (const auto& i : mobius::imagefile::get_imagefile_types ())
     std::cerr << "       " << i.type << "\t\t" << i.description << std::endl;
 
   std::cerr << std::endl;
@@ -49,7 +51,7 @@ usage ()
   std::cerr << "     output types are:" << std::endl;
   std::cerr << "       autodetect\ttry to autodetect imagefile type (default)" << std::endl;
 
-  for (auto i : mobius::imagefile::imagefile::typelist)
+  for (const auto& i : mobius::imagefile::get_imagefile_types ())
     if (i.is_writeable)
       std::cerr << "       " << i.type << "\t\t" << i.description << std::endl;
 
@@ -83,17 +85,20 @@ get_size (const std::string& text)
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get type from URL
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 get_type_from_url (const std::string& url)
 {
   std::string type;
 
-  mobius::uri uri (url);
+  mobius::io::uri uri (url);
   std::string extension = uri.get_extension ();
 
   if (extension == "001")
     type = "split";
 
+  else if (extension == "E01")
+    type = "ewf";
+
   else
     type = "raw";
 
@@ -103,10 +108,10 @@ get_type_from_url (const std::string& url)
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 //! \brief get output URL from input URL and type
 // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-const std::string
+std::string
 get_url_from_type (const std::string& input_url, const std::string& type)
 {
-  mobius::uri uri (input_url);
+  mobius::io::uri uri (input_url);
   std::string url = input_url.substr (0, input_url.length () - uri.get_extension ().length ());
 
   if (type == "raw")
@@ -208,7 +213,7 @@ main (int argc, char **argv)
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // check if input imagefile is available
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  mobius::imagefile::imagefile image_in (input_url, input_type_arg);
+  auto image_in = mobius::imagefile::new_imagefile_by_url (input_url, input_type_arg);
 
   if (!image_in.is_available ())
     {
@@ -221,15 +226,27 @@ main (int argc, char **argv)
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // create output imagefile
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
-  mobius::imagefile::imagefile image_out (output_url, output_type_arg);
-  image_out.set_segment_size (get_size (segment_size_arg));
+  auto image_out = mobius::imagefile::new_imagefile_by_url (output_url, output_type_arg);
+
+  if (image_out.get_type () == "ewf")
+    {
+      mobius::imagefile::imagefile_ewf image (image_out);
+      image.set_segment_size (get_size (segment_size_arg));
+      image.set_compression_level (1);
+    }
+
+  else if (image_out.get_type () == "split")
+    {
+      mobius::imagefile::imagefile_split image (image_out);
+      image.set_segment_size (get_size (segment_size_arg));
+    }
 
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   // copy imagefile
   // =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
   auto reader = image_in.new_reader ();
   auto writer = image_out.new_writer ();
-  constexpr decltype (reader)::size_type BLOCK_SIZE = 65536;
+  constexpr decltype (reader)::size_type BLOCK_SIZE = 1048576;
 
   std::cout << std::endl;
   std::cout << "About to copy " << reader.get_size () << " bytes" << std::endl;
@@ -247,4 +264,6 @@ main (int argc, char **argv)
       printf ("Copied %lu bytes\r", copied);
       data = reader.read (BLOCK_SIZE);
     }
+    
+  std::cout << std::endl;
 }
